import numpy as np
from numpy import linalg as LA
from scipy.optimize import fsolve
import matplotlib.pyplot as plt

def backward_euler_one_step (f, t, y, h):
    yn = y; err = 10000000
    while (err > 1e-14):
        yn1 = y + h * f(t + h, yn)
        err = LA.norm (yn1 - yn, 2)
        yn = yn1
    return yn1

def implicit_trapezoidal_one_step (f, t, y, h):
    aux = lambda yn :  y + h * 0.5 * (f(t, y) + f(t+h, yn)) - yn
    res = fsolve(aux, y)
    return res

def solve (f, t0, y0, tend, nsteps):
    h = (tend - t0) / nsteps
    time = np.linspace(t0, tend, nsteps)
    yn = y0; y = []
    for t in time:
        y.append(yn)
        yn = implicit_trapezoidal_one_step (f, t, yn, h)
    return [ time, y]

def dynamic (t, y):
    return np.array([-y[1], y[0]])

[t, y] = solve (dynamic, 0.0, np.array([1., 0.]), 2*np.pi*10, 100)

fig, ax = plt.subplots()
plt.plot(t, y)

plt.xlabel('time')
plt.ylabel('Solution y')
plt.title("Simple Plot")
plt.legend()
plt.show()

#yn = backward_euler_one_step (f, t, yn, h) # and 50000 steps
